/*
 * Copyright (c) 2000 David Flanagan.  All rights reserved.
 * This code is from the book Java Examples in a Nutshell, 2nd Edition.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book (recommended),
 * visit http://www.davidflanagan.com/javaexamples2.
 */
// package development.java.davidFlanagan.gui.uebungen;
import javax.swing.*;       // For JPanel component
import java.awt.*;          // For Graphics object
import java.awt.event.*;    // For Event and Listener objects

/**
 * A simple JPanel subclass that uses event listeners to allow the user
 * to scribble with the mouse.  Note that scribbles are not saved or redrawn.
 **/
public class ScribblePane extends JPanel
implements MouseListener, MouseMotionListener {
    
    //die aktuelle anzahl an gezeichneten geometrieobjekten
    public int anzFormen = 0;
    //ein array von geometrieobjekten
    private Form[] formen = new Form[1024];
    //nummer des zum verschieben selektierten objekts
    private int nrMov=1024;
    
    protected int last_x, last_y;  // Previous mouse coordinates
    
    //die aktuelle farbe
    private Color color = Color.gray;
    public void setColor(Color colorPar) {
        color = colorPar;
    }
    
    public Color getColor() {
        return color;
    }
        
    String werkzeug;
    //private Rechteck rechteck;
    
    public ScribblePane() {
	// This component registers itself as an event listener for
	// mouse events and mouse motion events.
	this.addMouseListener(this);
	this.addMouseMotionListener(this);
        //this.getGraphics().setColor(Color.blue);

	// Give the component a preferred size
	setPreferredSize(new Dimension(450,200));
    }

    // A method from the MouseListener interface.  Invoked when the
    // user presses a mouse button.
    public void mousePressed(MouseEvent e) {
        //die koordinaten des mauszeigers werden beim klicken auf die linke
        //maustaste in x und y gespeichert
        int x = e.getX();
        int y = e.getY();
        moveTo(x, y);
        //
        if (werkzeug == "Verschieben") {
            //erstmal alle ent-selektieren
            this.deSelektieren();
            //dann neu selektieren und merken, welches selektiert ist
            for(int i = 0; i < anzFormen; i++) {
                formen[i].setSelection(x,y);
                if (formen[i].getSelection()) nrMov = i;
            }
            this.neuZeichnen();
        }  
    }
    
    // A method from the MouseMotionListener interface.  Invoked when the
    // user drags the mouse with a button pressed.
    public void mouseDragged(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        if (werkzeug == "Stift") {
            lineTo(x,y);
        }
        if (werkzeug == "Verschieben") {
            //ursprungspunkt des objekts neu setzen
            formen[nrMov].setOrigin(x,y);
            //this.repaint(x,y,100,100);
            this.neuZeichnen();
        }
    }
    
    //hier soll bei einem mausklick in die pane eine figur gezeichnet werden
    //die figur haengt davon ab, welchen wert die eigenschaft werkzeug hat
    public void mouseClicked(MouseEvent e) {
        //die koordinaten des mauszeigers werden beim klicken auf die (linke)
        //maustaste in x und y gespeichert
        int x = e.getX();
        int y = e.getY();
        if (werkzeug == "Rechteck") {
            System.out.println("das ist das " + anzFormen + "-te element");
            //neues formen-objekt erzeugen
            formen[anzFormen] = 
            new Rechteck(x,y,50, 50, color);
            Rechteck test = new Rechteck(5,5,5,5,Color.yellow);
            System.out.println("test"+test);
            //koordinatensystem ermitteln
            Graphics g = getGraphics();
            //farbe setzen
            //g.setColor(color);
            //formen-objekt zeichnen
            formen[anzFormen].zeichnen(g);
            //anzahl formen-objekte um 1 erhoehen
            anzFormen++;
        }
        //
        if (werkzeug == "Zeiger") {
            for(int i = 0; i < anzFormen; i++) {
                formen[i].setSelection(x,y);
            }
            this.neuZeichnen();
        } 
        
        //wenn das werkzeug auf aendern gesetzt ist, erscheint beim anklicken 
        //eines objekts ein eingabedialog zur eingabe der neuen eigenschaften
        if (werkzeug == "Aendern") {
            //deselektieren und neues objekt waehlen
            int i;
            nrMov = 0;
            this.deSelektieren();
            for(i = 0; i < anzFormen; i++) {
                formen[i].setSelection(x,y);
                if (formen[i].getSelection()) nrMov = i;
            }
            //eingabedialog erzeugen, alle eigenschaften des 
            //gewaehlten objekts anzeigen und neue eigenschaften einlesen
            //und an methode setSplittedString(String) uebergeben 
            formen[nrMov].setSplittedString(
                             JOptionPane.showInputDialog(
                                 this, formen[nrMov].getAlle(), 
                                 "objekteigenschaften aendern", 
                                 JOptionPane.QUESTION_MESSAGE
                             )
            );
            formen[nrMov].parseSplittedString();
            this.deSelektieren();
            this.neuZeichnen();
        } //ende der aendern abfrage
          
    } //ende mouseClicked()
        
    // The other, unused methods of the MouseListener interface.
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    
    // The other, unused, method of the MouseMotionListener interface.
    public void mouseMoved(MouseEvent e) {}
    
    //hier die methoden zum zeichnen
    
    public void moveTo(int x , int y) {
        last_x = x;
        last_y = y;
    }
    
    public void lineTo(int x, int y) {
        Graphics g = getGraphics();
        g.setColor(color);
        g.drawLine(last_x, last_y, x, y);
        moveTo(x,y);
    }
    
    public void setWerkzeug(String werkzeugPar) {
        werkzeug = werkzeugPar;
        System.out.println("werkzeug auf " + werkzeug + " gesetzt");
    }
    
    public void neuZeichnen() {
       // System.out.println("erstmal loeschen:");
        Graphics g = getGraphics();
        //update(g);
        paint(g);
        //g.setColor(color);
        for(int i = 0; i < anzFormen; i++) {
            System.out.println("neuzeichnen von element nr. " + i);
            formen[i].zeichnen(g); 
        }
    }
    
    public void deSelektieren() {
        for(int i = 0; i < anzFormen; i++) {
            formen[i].setSelection(false);
        }
        this.neuZeichnen();
    }  
        
    public void loescheFormen() {
           anzFormen = 0;
           System.out.println("array-zeiger: " + anzFormen);
    }
    
    public Form[] getFormen() {
        return formen;
    }
    
    public void setFormen(Form[] formenVar) {
        formen = formenVar;
    }
    
}
